`timescale 1ns/1ps

module axis_fifo_tb;

parameter TDATA_SIZE = 16;
parameter TUSER_SIZE = 2;
parameter DEPTH_LOG = 5;
parameter SRC_CLK_PERIOD = 5.55;
parameter DST_CLK_PERIOD = 7.77;
parameter INPUT_FILE = "input.bin";
parameter OUTPUT_FILE = "output.bin";

reg ARESETN;

reg S_AXIS_ACLK;
wire S_AXIS_TVALID;
wire [TUSER_SIZE-1:0] S_AXIS_TUSER;
wire [TDATA_SIZE-1:0] S_AXIS_TDATA;
wire S_AXIS_TLAST;
wire S_AXIS_TREADY;

reg M_AXIS_ACLK;
wire M_AXIS_TVALID;
wire [TUSER_SIZE-1:0] M_AXIS_TUSER;
wire [TDATA_SIZE-1:0] M_AXIS_TDATA;
wire M_AXIS_TLAST;
wire M_AXIS_TREADY;

axis_async_fifo
#(
    .TDATA_SIZE(TDATA_SIZE),
    .TUSER_SIZE(TUSER_SIZE),
    .DEPTH_LOG(DEPTH_LOG)
) dut (
    .aresetn(ARESETN),

    .s_axis_aclk(S_AXIS_ACLK),
    .s_axis_tvalid(S_AXIS_TVALID),
    .s_axis_tuser(S_AXIS_TUSER),
    .s_axis_tdata(S_AXIS_TDATA),
    .s_axis_tlast(S_AXIS_TLAST),
    .s_axis_tready(S_AXIS_TREADY),

    .m_axis_aclk(M_AXIS_ACLK),
    .m_axis_tvalid(M_AXIS_TVALID),
    .m_axis_tuser(M_AXIS_TUSER),
    .m_axis_tdata(M_AXIS_TDATA),
    .m_axis_tlast(M_AXIS_TLAST),
    .m_axis_tready(M_AXIS_TREADY)
);

axis_slave_bfm
#(
    .OUTPUT_FILE(OUTPUT_FILE),
    .TUSER_SIZE(TUSER_SIZE),
    .TDATA_SIZE(TDATA_SIZE)
) slave (
    .aresetn(ARESETN),

    .s_axis_aclk(M_AXIS_ACLK),
    .s_axis_tvalid(M_AXIS_TVALID),
    .s_axis_tuser(M_AXIS_TUSER),
    .s_axis_tdata(M_AXIS_TDATA),
    .s_axis_tlast(M_AXIS_TLAST),
    .s_axis_tready(M_AXIS_TREADY)
);

axis_master_bfm
#(
    .INPUT_FILE(INPUT_FILE),
    .TDATA_SIZE(TDATA_SIZE),
    .TUSER_SIZE(TUSER_SIZE)
) master (
    .aresetn(ARESETN),

    .m_axis_aclk(S_AXIS_ACLK),
    .m_axis_tvalid(S_AXIS_TVALID),
    .m_axis_tuser(S_AXIS_TUSER),
    .m_axis_tdata(S_AXIS_TDATA),
    .m_axis_tlast(S_AXIS_TLAST),
    .m_axis_tready(S_AXIS_TREADY)
);

//generate clocks of desired frequency
initial begin
    S_AXIS_ACLK = 0;
    forever #(SRC_CLK_PERIOD/2) S_AXIS_ACLK = ~S_AXIS_ACLK;
end
initial begin
    M_AXIS_ACLK = 0;
    forever #(DST_CLK_PERIOD/2) M_AXIS_ACLK = ~M_AXIS_ACLK;
end

//generate reset
initial begin
    ARESETN = 1;
    #1000
    ARESETN = 0;
    #1000
    ARESETN = 1;
    @(posedge master.write_done);
    @(posedge slave.read_timeout);
    $finish;
end

endmodule
