module axis_master_bfm
#(
    parameter INPUT_FILE = "",
    parameter TDATA_SIZE = 16,
    parameter TUSER_SIZE = 2,
    parameter ENDIANNESS = "little"
)
(
    input                       aresetn,
    /*
     * AXI Stream output
     */
    input                       m_axis_aclk,
    output reg [TDATA_SIZE-1:0] m_axis_tdata,
    output reg                  m_axis_tvalid,
    input                       m_axis_tready,
    output reg                  m_axis_tlast,
    output reg [TUSER_SIZE-1:0] m_axis_tuser,

    output                      done
);

reg [7:0] nwrites;
integer write_done = 0;
integer input_file = 0;
reg [7:0] read_data;

assign done = (write_done == 1);

initial begin
    //open output file
    input_file = $fopen(INPUT_FILE,"r");
    if(input_file == 0) begin
        $display("AXI Stream Master BFM: Unable to open input file");
        $finish;
    end else
        $display("AXI Stream Master BFM: Opened input file %s",INPUT_FILE);

    //wait until data transfer ends
    @(posedge write_done);
    #100

    //clean up and end simulation
    $fclose(input_file);
end

initial begin
    write_done = 0;
    m_axis_tvalid = 0;
    m_axis_tdata = 0;
    m_axis_tuser = 0;
    m_axis_tlast = 0;
    @(negedge aresetn);
    @(posedge aresetn);
    while(!$feof(input_file)) begin
        //wait a random amount of clock cycles (1 to 64)
        m_axis_tvalid = 0;
        repeat(1 + ({$random()} % 64)) @(posedge m_axis_aclk);
        nwrites = 1 + ({$random()} % 128);
        while((nwrites > 0) & !$feof(input_file)) begin
            repeat(TDATA_SIZE/8) begin
                $fread(read_data,input_file);
                if(ENDIANNESS == "little")
                    m_axis_tdata = {read_data,m_axis_tdata[TDATA_SIZE-1:8]};
                else
                    m_axis_tdata = {m_axis_tdata[TDATA_SIZE-8-1:0],read_data};
            end
            if(!$feof(input_file)) begin
                m_axis_tvalid = 1;
            end else begin
                m_axis_tvalid = 0;
                write_done = 1;
            end
            if(m_axis_tready) @(posedge m_axis_aclk); //wait one clock cycle if ready
            while(~m_axis_tready) @(posedge m_axis_aclk);//otherwise, wait as many cycles as needed until ready
            nwrites = nwrites-1;
            m_axis_tvalid = 0;
        end
    end
end

endmodule
