module axis_monitor
#(
    parameter OUTPUT_FILE = "",
    parameter TDATA_SIZE = 16,
    parameter TUSER_SIZE = 2,
    parameter ENDIANNESS = "little"
)
(
    input                   aresetn,
    /*
     * AXI Stream input
     */
    input                   s_axis_aclk,
    input  [TDATA_SIZE-1:0] s_axis_tdata,
    input                   s_axis_tvalid,
    input                   s_axis_tready,
    input                   s_axis_tlast,
    input  [TUSER_SIZE-1:0] s_axis_tuser,

    output                  done
);

wire read_timeout;
integer read_timeout_cnt = 0;
reg [7:0] nwait;
integer output_file = 0;
reg [TDATA_SIZE-1:0] write_data;

assign done = read_timeout;

initial begin
    //open output file
    output_file = $fopen(OUTPUT_FILE,"w");
    if(output_file == 0) begin
        $display("AXI Stream Monitor: Unable to open output file");
        $finish;
    end else
        $display("AXI Stream Monitor: Opened output file %s",OUTPUT_FILE);

    //wait until data transfer ends
    @(posedge read_timeout);
    $display("AXI Stream Monitor: Timed out, closing output file");
    #100

    //clean up and end simulation
    $fclose(output_file);
end

//write data process
always @(negedge s_axis_aclk)
    if(s_axis_tready & s_axis_tvalid) begin
        write_data = s_axis_tdata;
        repeat(TDATA_SIZE/8) begin
            if(ENDIANNESS == "little") begin
                $fwriteb(output_file,"%c",write_data[7:0]);
                write_data = write_data >> 8;
            end else begin
                $fwriteb(output_file,"%c",write_data[TDATA_SIZE-1:TDATA_SIZE-8]);
                write_data = write_data << 8;
            end
            $fflush(output_file);
        end
    end

always @(posedge s_axis_aclk)
    if((s_axis_tready & s_axis_tvalid) | ~aresetn)
        read_timeout_cnt <= 0;
    else
        read_timeout_cnt <= read_timeout_cnt + 1;

assign read_timeout = read_timeout_cnt >= 1000;

endmodule
